@@ -25,6 +25,10 @@
 #include <X11/extensions/Xfixes.h> /* Used to *really* hide cursor */
 #endif
 
+#ifdef HAVE_XCURSOR
+#include <X11/Xcursor/Xcursor.h>
+#endif
+
 #ifdef USE_XSETTINGS
 static void wm_xsettings_notify_cb (const char       *name,
 				    XSettingsAction   action,
@@ -132,7 +136,6 @@ wm_new(int argc, char **argv)
      w->img_generic_icon = mb_pixbuf_img_new_from_file(w->pb, GENERIC_ICON);
      if (w->img_generic_icon == NULL)
        {
-	 fprintf(stderr,"matchbox: WARNING: failed to load %s . Disabling icons.\n", GENERIC_ICON); 
 	 w->config->use_icons = 0;
        }
      else misc_scale_wm_app_icon(w);
@@ -1374,8 +1377,12 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
    if (!(value_mask & CWWidth))  req_w = c->width;
    if (!(value_mask & CWHeight)) req_h = c->height;
    
-   if (e->x <= 0 || !(value_mask & CWX)) req_x = c->x;
-   if (e->y <= 0 || !(value_mask & CWY)) req_y = c->y;
+   if ((e->x <= 0 && !(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
+	|| !(value_mask & CWX)) 
+       req_x = c->x;
+   if ((e->y <= 0 && !(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
+       || !(value_mask & CWY)) 
+     req_y = c->y;
 
    dbg("%s() for win %s - have w: %i vs %i, h: %i" 
        "vs %i, x: %i vs %i, y: %i vs %i,\n", 
@@ -1485,6 +1492,7 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
         * - eg toolbar/panel/input windows may dissapear and make
         *      more space available. 
         */
+
        if (e->width && (value_mask & CWWidth) 
 	   && e->width != c->width && e->width != c->init_width)
 	 c->init_width = e->width;
@@ -1536,14 +1544,66 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	   dialog_constrain_geometry(c, &req_x, &req_y, &req_w, &req_h);
 
 #ifdef USE_ALT_INPUT_WIN
-	if (c->flags & (CLIENT_TB_ALT_TRANS_FOR_DIALOG|
-				CLIENT_TB_ALT_TRANS_FOR_APP))
-	{
-		req_w = c->init_width = c->width;
-		req_x = c->x;
-	}
+	 /* Alternate Input windows use the dialog type to 
+	  * fit in with stacking etc but they cant change width, x nor y
+	  */
+	 if (c->flags & (CLIENT_TB_ALT_TRANS_FOR_DIALOG
+	                   |CLIENT_TB_ALT_TRANS_FOR_APP))
+	   {
+	     if (c->height != req_h)
+	       {
+		 Client *trans_client = c->trans;
+		 int     diff = c->height - req_h;
+
+		 c->height  = req_h;
+		 c->y      += diff;
+		 toolbar_client_move_resize(c);
+
+		 if (trans_client)
+		   {
+		     if (trans_client->type == MBCLIENT_TYPE_DIALOG)
+		       {
+			 /* do not grow over their initial size
+			    because of IM resizing smaller */
+			 if (trans_client->height + diff 
+			              > trans_client->init_height)
+			   {
+			     trans_client->height = trans_client->init_height;
+			   }
+			 else if (diff < 0)
+			   {
+			     /* Input window has grown larger so push it 
+			      *	up but keep it on screen.
+			     */
+			     int x, y, w, h;
+			     dialog_client_get_coverage(trans_client, 
+							&x, &y, &w, &h);
+
+			     if (c->y < y+h)
+			       {
+				 int amount = (y+h) - c->y;
+				 
+				 /* move upwards */
+				 trans_client->y -= amount;
+				 /* but not over screen area */
+				 if (y-amount < 0)
+				   {
+				     trans_client->height -= amount-y;
+				     trans_client->y += amount-y;
+				   }
+			       }
+			   }
+		       }
+		     else 	/* Application window */
+		       trans_client->height += diff;
+
+		     trans_client->move_resize(trans_client);
+		     trans_client->redraw(trans_client, False);
+		   }
+	       }
+	     return;
+	   }
 #endif
-	   
 	   if (c->width == req_w && c->height == req_h)
 	     want_fake_configure = True; 
 
@@ -1551,9 +1611,6 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	   if (c->width != req_w)
 	     client_buttons_delete_all(c);
 
-	   /* TODO: fix composite for this */
-	   /* comp_engine_client_hide(c->wm, c);*/
-	   
 	   xwc.width  = c->width  = req_w;
 	   xwc.height = c->height = req_h;
 	   xwc.x      = c->x      = req_x;
@@ -1867,7 +1924,7 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
 
       client_get_wm_protocols(c);
 
-      dbg("%s() WM_PROTOCOLS changed, flags now %i\n", __func__, c->flags);
+      dbg("%s() WM_PROTOCOLS changed, flags now %li\n", __func__, c->flags);
 
       /* If flags have changed, likely means a WM_CONTEXT_HELP|etc has changed 
        * and thus titlebar needs a repaint.
@@ -1971,6 +2028,9 @@ wm_make_new_client(Wm *w, Window win)
 		 {
 		   dbg("%s() got toolbar atom\n", __func__ );
 		   c = toolbar_client_new(w, win);
+#ifdef USE_ALT_INPUT_WIN
+		   c->flags &= ~CLIENT_IS_MINIMIZED;
+#endif
 		   if (c == NULL) goto end;
 		 }
 	       else if (value[0] == w->atoms[WINDOW_TYPE_DESKTOP])
@@ -1986,21 +2046,24 @@ wm_make_new_client(Wm *w, Window win)
 		   if (c == NULL) goto end;
 		   c->flags |= (CLIENT_TITLE_HIDDEN_FLAG|CLIENT_IS_SPLASH_WIN);
 		 }
-	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG]
-			|| value[0] == w->atoms[WINDOW_TYPE_MENU])
+	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG])
 		 {
 		   dbg("%s() got type dialog atom\n", __func__ );
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
-
-		   /* check to see if we're a 'menu' dialog  */
-		   if (value[0] == w->atoms[WINDOW_TYPE_MENU])
-		     {
-		       dbg("%s() Got menu hint\n", __func__);
-		       c->flags |= CLIENT_IS_MENU_DIALOG;
-		     }
 		 }
-	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE])
+	       else if (value[0] == w->atoms[WINDOW_TYPE_MENU]
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_POPUP_MENU]
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_DROPDOWN_MENU])
+		 {
+		   dbg("%s() got type menu atom\n", __func__ );
+		   c = dialog_client_new(w, win, NULL);
+		   if (c == NULL) goto end;
+		   c->flags |= CLIENT_IS_MENU_DIALOG;
+		 }
+	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE] 
+			/* Above to depricate for below */
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_NOTIFICATION])
 		 {
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
@@ -2027,7 +2090,7 @@ wm_make_new_client(Wm *w, Window win)
        if (c) 
 	 c->flags ^= mwm_flags;
 
-       dbg("%s() got MWM flags: %i\n", __func__, c->flags );
+       dbg("%s() got MWM flags: %li\n", __func__, c->flags );
      }
 
    /* check for transient - ie detect if its a dialog */
@@ -2076,6 +2139,12 @@ wm_make_new_client(Wm *w, Window win)
       else if (c->type == MBCLIENT_TYPE_DIALOG) /* already exists, update  */
 	c->trans = t;  		/* TODO: what about other types 
 				         being transient for things ?*/
+
+      /* Make sure above state is inherited if parent has it */
+      if (c->trans != NULL
+	  && c->trans->type == MBCLIENT_TYPE_DIALOG 
+	  && c->trans->flags & CLIENT_HAS_ABOVE_STATE)
+	c->flags |= CLIENT_HAS_ABOVE_STATE;
    }
    
    if (c == NULL) /* Noting else found, default to a main client */
@@ -2107,6 +2176,9 @@ wm_make_new_client(Wm *w, Window win)
    
    c->move_resize(c);          	/* set pos + size */
 
+   /* send new configuration to client - needed */
+   client_deliver_config(c);
+
    /* TODO:
     *
     * Its likely the size we given the new client, is not what it requested. 
@@ -2124,15 +2196,32 @@ wm_make_new_client(Wm *w, Window win)
     * Note, this seems worst on GTK apps.
     */
 
+   XGrabButton(c->wm->dpy, Button1, 0, c->window, True, ButtonPressMask,
+	       GrabModeSync, GrabModeSync, None, None);
+
+   /* Handle an application started iconized */
+   if (c->flags & CLIENT_IS_MINIMIZED
+       && c->type == MBCLIENT_TYPE_APP)
+     {
+       /* Clear the flag now to be safe */
+       c->flags &= ~CLIENT_IS_MINIMIZED;
+
+       c->redraw(c, False);		/* draw the decorations ready */
+       c->iconize (c);
+
+       ewmh_update_lists(w); 
+
+       XUngrabServer(w->dpy);
+       XFlush(w->dpy);
+
+       goto end;
+     }
+
    dbg("%s() showing new client\n", __func__);
 
    c->redraw(c, False);		/* draw the decorations ready */
-
    wm_activate_client(c);       /* Map it into stack, ( will call show()) */
 
-   XGrabButton(c->wm->dpy, Button1, 0, c->window, True, ButtonPressMask,
-	       GrabModeSync, GrabModeSync, None, None);
-
    /* Let window know were all done */
 
    ewmh_state_set(c); 		/* XXX This is likely not needed */
@@ -2417,6 +2506,7 @@ wm_update_layout(Wm         *w,
    {
      if (p->type == MBCLIENT_TYPE_DIALOG) 
        {
+	 Bool force_height = False;
 	 int req_x = p->x, req_y = p->y, req_w = p->width, req_h = p->height;
 
 #ifdef USE_ALT_INPUT_WIN
@@ -2432,8 +2522,20 @@ wm_update_layout(Wm         *w,
 	     client_deliver_config(p);
 	     continue;
 	   }
+#else
+	 /* Always try and back to intially requested height, a  
+          * panel or input win may have dissapeared giving more 
+          * space.
+          * NOTE: we avoid doing this for ALT_INPUT_WINS (maemo)
+	 */
+	 if (p->init_height && p->init_height > req_h)
+	   {
+	     req_h = p->init_height;
+	     force_height = True;
+	   }
 #endif
-	 if (!dialog_constrain_geometry(p, &req_x, &req_y, &req_w, &req_h))
+	 if (!dialog_constrain_geometry(p, &req_x, &req_y, &req_w, &req_h)
+	     || force_height)
 	   {
 	     p->x = req_x; p->y = req_y; p->width = req_w; p->height = req_h;
 	     p->move_resize(p);
@@ -2521,7 +2623,6 @@ wm_activate_client(Client *c)
       /* As matchbox works around 'main' windows ( apps/main and desktop wins).
 	 We need to sync extra stuff up when displaying a new one.
        */
-
       Bool switching_from_to_fullscreen = False;
 
       /* save focus state for transient dialogs of prev showing main win */
@@ -2688,7 +2789,6 @@ wm_activate_client(Client *c)
       && mbtheme_has_titlebar_panel(w->mbtheme)
       && !(w->have_titlebar_panel->flags & CLIENT_DOCK_TITLEBAR_SHOW_ON_DESKTOP))
     {
-      // stack_move_below_client(c, w->client_desktop);
       /* TODO: Change to stack move bottom ? 
        *
        * eg use stack_move_above_client(c, NULL)
@@ -2701,8 +2801,6 @@ wm_activate_client(Client *c)
   if (set_desktop_show_hint) 
     ewmh_update_desktop_hint(w);
 
-  stack_dump(w);
-
   dbg("%s() now syncing above window stack\n", __func__);
 
   stack_sync_to_display(w);
@@ -2716,6 +2814,8 @@ wm_activate_client(Client *c)
 
   client_set_focus(client_to_focus); /* set focus if needed and ewmh active */
 
+  stack_dump(w);
+
   if (set_current_app_active_hint)
     ewmh_set_current_app_window(w);
 
@@ -2927,6 +3027,8 @@ wm_get_desktop(Wm *w)
 #define XSET_LOWLIGHT   3
 #define XSET_TITLEBARS  4
 #define XSET_COMPOSITE  5
+#define XSET_CURSOR_THEME_NAME 6
+#define XSET_CURSOR_THEME_SIZE 7
 
 static void
 wm_xsettings_notify_cb (const char       *name,
@@ -2944,14 +3046,15 @@ wm_xsettings_notify_cb (const char       *name,
     { "MATCHBOX/CURSOR",    XSET_CURSOR    },
     { "MATCHBOX/TITLEBARS", XSET_TITLEBARS }, /* XXX Not implemeted */
     { "MATCHBOX/COMPOSITE", XSET_COMPOSITE },
+    { "Gtk/CursorThemeName", XSET_CURSOR_THEME_NAME },
+    { "Gtk/CursorThemeSize", XSET_CURSOR_THEME_SIZE },
     { NULL,       -1 } 
   };
 
   while(  mb_xsettings[i].name != NULL )
     {
       if (!strcmp(name, mb_xsettings[i].name)
-	  && setting != NULL 	/* XXX set to NULL when action deleted */
-	  && setting->type == XSETTINGS_TYPE_STRING )
+	  && setting != NULL) 	/* Set to NULL when action deleted */
 	{
 	  key = mb_xsettings[i].value;
 	  break;
@@ -2961,44 +3064,66 @@ wm_xsettings_notify_cb (const char       *name,
     
   if (key == XSET_UNKNOWN) return;
 
-  switch (action)
+  if (setting->type == XSETTINGS_TYPE_STRING)
     {
-    case XSETTINGS_ACTION_NEW:
-    case XSETTINGS_ACTION_CHANGED:
-      switch (key)
+      switch (action)
 	{
-	case XSET_COMPOSITE:
-	  if (!strcasecmp("off", setting->data.v_string)
-	      || !strcasecmp("false", setting->data.v_string))
+	case XSETTINGS_ACTION_NEW:
+	case XSETTINGS_ACTION_CHANGED:
+	  switch (key)
 	    {
-	      comp_engine_deinit(w);
-	    }
-	  else
-	    { 
-	      comp_engine_reinit(w);
+	    case XSET_COMPOSITE:
+	      if (!strcasecmp("off", setting->data.v_string)
+		  || !strcasecmp("false", setting->data.v_string))
+		{
+		  comp_engine_deinit(w);
+		}
+	      else
+		{ 
+		  comp_engine_reinit(w);
+		}
+	      break;
+	    case XSET_THEME:
+	      if (w->flags & STARTUP_FLAG)
+		w->config->theme = strdup(setting->data.v_string);
+	      else
+		mbtheme_switch(w, setting->data.v_string);
+	      break;
+	    case XSET_CURSOR:
+	      if (!strcasecmp("true", setting->data.v_string))
+		wm_set_cursor_visibility(w, True);
+	      else 
+		wm_set_cursor_visibility(w, False);
+	      break;
+	    case XSET_TITLEBARS:
+	      /* XXX todo */
+	      break;
+	    case XSET_CURSOR_THEME_NAME:
+#ifdef HAVE_XCURSOR
+	      XcursorSetTheme (w->dpy, setting->data.v_string);
+#endif
+	      break;
+	    default:
+	      break;
 	    }
+	case XSETTINGS_ACTION_DELETED:
+	  /* Do nothing for now */
 	  break;
-	case XSET_THEME:
-	  if (w->flags & STARTUP_FLAG)
-	      w->config->theme = strdup(setting->data.v_string);
-	  else
-	      mbtheme_switch(w, setting->data.v_string);
-	  break;
-	case XSET_CURSOR:
-	  if (!strcasecmp("true", setting->data.v_string))
-	    wm_set_cursor_visibility(w, True);
-	  else 
-	    wm_set_cursor_visibility(w, False);
-	  break;
-	case XSET_TITLEBARS:
-	  /* XXX todo */
-	  break;
-
 	}
-    case XSETTINGS_ACTION_DELETED:
-      /* Do nothing for now */
-      break;
     }
+  else if (setting->type == XSETTINGS_TYPE_INT)
+    {
+#ifdef HAVE_XCURSOR
+      if ( (action == XSETTINGS_ACTION_NEW 
+	    || action == XSETTINGS_ACTION_CHANGED) 
+	   && key == XSET_CURSOR_THEME_NAME)
+	{
+	  if (setting->data.v_int)
+	    XcursorSetDefaultSize (w->dpy, setting->data.v_int);
+	}
+#endif
+    }
+
 }
 
 #endif
